  typedef struct wfh_struct WFH;


  typedef int (*wfh_compare)(const char* word1,
                             const char* word2);
   /* Comparison function for words. The words   */
   /* are always valid strings. The return value */
   /* is interpreted as:                         */
   /*                                            */
   /* 0: word1 equals word2.                     */
   /* >0: word1 is greater than word2            */
   /* <0: word1 is greater than word2            */


  typedef int (*wfh_visitor)(const char* word,
                             unsigned freq,
			     void* commondata);
  /* Function for visiting all words in histogram */
  /* in their sorted order.                       */
  /* word is always a valid string                */
  /* freq is the frequency of the word.           */
  /* commondata is the pointer passed to the      */
  /* traverse function.                           */
  /* A return value of 0 terminates the traversal */
  /* of words.                                    */


  WFH* wfh_create(wfh_compare comp);
  /* Create a new word frequency histogram.     */
  /*                                            */
  /* Return value:                              */
  /*   A valid pointer to a word frequency hist.*/
  /*   0 on failure.                            */
  /*                                            */
  /* Precondition:                              */
  /*   comp != 0                                */


  unsigned wfh_insert(WFH* hist,
                      const char* word);
  /* Insert a word in the histogram.            */
  /*                                            */
  /* Return value:                              */
  /*   The frequency of the word.               */
  /*   0 on failure (invalidates postcond)      */
  /*                                            */
  /* Preconditions:                             */
  /*   hist != NULL                             */
  /*   word != NULL                             */
  /*                                            */
  /* Postconditions:                            */
  /* wfh_find(hist,word) ==                      */
  /*   1+old wfh_find(hist,word)                 */


  unsigned wfh_find(WFH* hist, const char* word);
  /* Find frequency of a word in histogram.     */
  /*                                            */
  /* Return values:                             */
  /*   The frequency of word.                   */
  /*   0 if word not in hist.                   */
  /*                                            */
  /* Preconditions:                             */
  /*   hist != NULL                             */
  /*   word != NULL                             */


  int wfh_traverse(WFH* hist,
		   wfh_visitor visitor,
		   void* commondata);
  /* Visit all words in histogram in their      */
  /* sorted order.                              */
  /*                                            */
  /* Return values:                             */
  /*   1, all words visited.                    */
  /*   0, traversal aborted by visotor.         */
  /*                                            */
  /* Preconditions:                             */
  /*   hist != NULL                             */
  /*   visitor != NULL                          */

  void wfh_destroy(WFH* hist);
  /*  Destroy the histogram.                    */
  /*                                            */
  /* Preconditions:                             */
  /*   hist != NULL                             */
