  #include <stdio.h>
  #include <string.h> /* strcmp */
  #include "wordfile.h"
  #include "wfh.h"


  int printAll(const char* word, 
	       unsigned freq,
	       void* common)
  {
    int width = *(int*)common;
    printf("%*d:%s\n", width, freq, word);
    /* %* means "interpret the parameter on this place */
    /* as an integer specifying the width. For example:*/
    /* printf("%5d", 12); yields the same result as    */
    /* printf("%*d", 5, 12); In the latter case,       */
    /* though, the width can be specified by a variable*/
    /* and thus determined at runtime, as opposed to   */
    /* the former case, where the width must be known  */
    /* at compile time.                                */
    return 1;
  }

  int main(int argc, char* argv[])
  {
    WORDFILE* file;
    WFH* freqhist;
    int width = 4;
    int argcount = 1;
    char* filename;
    char word[80];/* what language has this long words? */

    if (argc < 2)
    {
      printf("Usage: wordfreq filename(s)\n");
      return 1;
    }

    for (filename = argv[argcount]; 
	 filename != NULL;
	 filename = argv[++argcount])
    {
      /* open our wordfile, or exit on failure */
      file = wordfile_open(filename);
      if (file == NULL)
      {
        fprintf(stderr, "Couldn't open \"%s\"\n", filename);
        continue;
      }

      /* OK, now create the word frequency histogram, or */
      /* exit on failure.                                */
      freqhist = wfh_create(strcmp);
      if (freqhist == NULL)
      {
        printf("Failed creating histogram\n");
        return 2;
      }

      /* Read words from the file, one by one, and insert */
      /* into the histogram, until all words have been    */
      /* read                                             */
      while (wordfile_nextword(file,word,sizeof(word))>0)
      {
        wfh_insert(freqhist, word);
      }

      /* when done reading, close the file */
      wordfile_close(file);

      /* Now when the histogram is complete for the */
      /* just read file, print the frequencies.     */

      printf("\nFrequencies of the words in \"%s\"\n",
             filename);
      wfh_traverse(freqhist, printAll, (void*)&width);

      /* And when done with that, destroy the histogram */
      wfh_destroy(freqhist);
    }
    return 0;
  }


